// *** bitmap for Rockbox plugins *** by eli_sherer

#include <stdio.h>
#include <stdlib.h>
#include "plugin.h"

//*********************************PROCESSOR STUFF*****************************
#ifdef ROCKBOX_LITTLE_ENDIAN
	#define readshort(x) *(x)
	#define readlong(x) *(x)
#else

static short readshort(short *value) {
    unsigned char* bytes = (unsigned char*) value;
    return bytes[0] | (bytes[1] << 8);
}
static long readlong(long *value) {
    unsigned char* bytes = (unsigned char*) value;
    return (long)bytes[0] | ((long)bytes[1] << 8) |
        ((long)bytes[2] << 16) | ((long)bytes[3] << 24);
}

#endif

//*********************************BMP STUFF*****************************
#ifdef __GNUC__
#define STRUCT_PACKED __attribute__((packed))
#else
#define STRUCT_PACKED
#pragma pack (push, 2)
#endif 

struct Fileheader {
    unsigned char magicB,magicM;/* signature - 'BM' */
    unsigned long Size;         /* file size in bytes */
    unsigned short Reserved1;   /* 0 */
    unsigned short Reserved2;   /* 0 */
    unsigned long OffBits;      /* offset to bitmap */
    unsigned long StructSize;   /* size of this struct (40) */
    unsigned long Width;        /* bmap width in pixels */
    unsigned long Height;       /* bmap height in pixels */
    unsigned short Planes;      /* num planes - always 1 */
    unsigned short BitCount;    /* bits per pixel */
    unsigned long Compression;  /* compression flag */
    unsigned long SizeImage;    /* image size in bytes */
    long XPelsPerMeter;         /* horz resolution */
    long YPelsPerMeter;         /* vert resolution */
    unsigned long ClrUsed;      /* 0 -> color table size */
    unsigned long ClrImportant; /* important color count */
} STRUCT_PACKED; 

struct RGBTRIPLE { /* reveresed */
  unsigned char Blue; 
  unsigned char Green; 
  unsigned char Red; 
} STRUCT_PACKED; 

struct RGBQUAD { /* reveresed */
  unsigned char Blue; 
  unsigned char Green; 
  unsigned char Red;
  unsigned char Reserved;
} STRUCT_PACKED; 


//*********************************LOAD FUNCTION*****************************

int divide(int a,int b)
{
	return ((a-(a%b))/b);
}

int retbit(int bit,int num)
{ int mult=128,temp;
	switch (bit)
	{ case 7: mult=1; break;
          case 6: mult=2;break;
          case 5: mult=4;break;
          case 4: mult=8;break;
          case 3: mult=16;break;
          case 2: mult=32;break;
          case 1: mult=64;break;
          case 0: mult=128;break;
	}
	temp=num&mult; /*now temp contains the specific bit it's place */
	temp=temp>>(7-bit);
	temp&=0x01;
	return temp;
}
int loadbitmap(const struct plugin_api* rb,int x0,int y0,char* filename)
{
    struct Fileheader fh;
    struct RGBTRIPLE tempt;
    struct RGBQUAD tempq, pal1bit[2], pal4bit[16],pal8bit[256];
    int fd,ret,x=0,y=0,i,j,k;
    int bitmap_width, bitmap_height;
    unsigned char temp,bitsbuf[4];
    //char *buf2bit; /* for 2 bit BMPs */

    fd = rb->open(filename, O_RDONLY); 
    if (fd<0)
    {   rb->splash(HZ*2, true, "Err: not found");
        return 0;
    }

    rb->read(fd, &fh, sizeof(struct Fileheader));

    bitmap_height = readlong(&fh.Height);
    bitmap_width = readlong(&fh.Width);

    switch (readshort(&fh.BitCount))
    {
        case 1: /* black and white */
            rb->lseek(fd, 54, SEEK_SET); /* offset of palette */
            ret = rb->read(fd,pal1bit,sizeof(struct RGBQUAD)*readlong(&fh.ClrUsed));
            rb->lseek(fd, (off_t)readlong(&fh.OffBits), SEEK_SET);
	    for (i = 0;i < (divide(bitmap_width,32)+1) * bitmap_height; i++,x++)
            {
                if (x==bitmap_width) { x=0; y++; }
                ret = rb->read(fd, bitsbuf,sizeof(unsigned char)*4);
                if (ret<0)
                {   rb->splash(HZ*2, true, "Err: wrong data");
                    rb->close(fd); 
                    return 0;
                }
                for(j = 0 ; j < 4 ; j++,x++)
                {   temp=bitsbuf[j];
                    for (k = 0; k < 7 ; k++)
                    {   if (x+1<bitmap_width)
                        {   rb->lcd_set_foreground(LCD_RGBPACK(pal1bit[retbit(k,temp)].Red,
                                pal1bit[retbit(k,temp)].Green,pal1bit[retbit(k,temp)].Blue));
                            rb->lcd_drawpixel(x0+x,y0+(bitmap_height-1)-y);
			    x++;
                        }
                        else break; /* stop loop */
                    }
                    if (x+1>=bitmap_width) break;
                }x--;
            }
            break;

        case 4: /* 16 colors */
            rb->lseek(fd, 54, SEEK_SET); /* offset of palette */
            ret = rb->read(fd,pal4bit,sizeof(struct RGBQUAD)*readlong(&fh.ClrUsed));
            rb->lseek(fd, (off_t)readlong(&fh.OffBits), SEEK_SET);
	    for (i = 0;i < (divide(bitmap_width,4)+1) * bitmap_height; i++,x++)
            {
                if (x==bitmap_width) { x=0; y++; }
                ret = rb->read(fd, bitsbuf,sizeof(unsigned char)*4);
                if (ret<0)
                {   rb->splash(HZ*2, true, "Err: wrong data");
                    rb->close(fd); 
                    return 0;
                }
                for(j = 0 ; j < 4 ; j++,x++)
                {   temp=bitsbuf[j];
                    rb->lcd_set_foreground(LCD_RGBPACK(pal4bit[(temp>>4)& 0x0f].Red,
                        pal4bit[(temp>>4)& 0x0f].Green,pal4bit[(temp>>4)& 0x0f].Blue));
                    rb->lcd_drawpixel(x0+x,y0+(bitmap_height-1)-y);
                    if (x+1<bitmap_width)
                    {   x++;
                        rb->lcd_set_foreground(LCD_RGBPACK(pal4bit[temp & 0x0f].Red,
                            pal4bit[temp & 0x0f].Green,pal4bit[temp & 0x0f].Blue));
                        rb->lcd_drawpixel(x0+x,y0+(bitmap_height-1)-y);
                    }
                    else break; /* stop loop */
                }x--;
            }
            break;

        case 8: /* 256 colors (no compression support yet) */
            rb->lseek(fd, 54, SEEK_SET); /* offset of palette */
            ret = rb->read(fd,pal8bit,sizeof(struct RGBQUAD)*readlong(&fh.ClrUsed));
            rb->lseek(fd, (off_t)readlong(&fh.OffBits), SEEK_SET);
	    for (i = 0; i < bitmap_width * bitmap_height; i++,x++)
            {
                if (x==bitmap_width) { x=0; y++; }
                ret = rb->read(fd, &temp,sizeof(unsigned char));
                if (ret<0)
                {   rb->splash(HZ*2, true, "Err: wrong data");
                    rb->close(fd); 
                    return 0;
                }
                rb->lcd_set_foreground(LCD_RGBPACK(pal8bit[temp].Red,pal8bit[temp].Green,pal8bit[temp].Blue));
                rb->lcd_drawpixel(x0+x,y0+(bitmap_height-1)-y);
            }
            break;

        case 24: /* 16mil colors */
            rb->lseek(fd, (off_t)readlong(&fh.OffBits), SEEK_SET); 
	    for (i = 0; i < bitmap_width * bitmap_height; i++,x++)
            {
                if (x==bitmap_width) { x=0; y++; }
		ret = rb->read(fd, &tempt,sizeof(struct RGBTRIPLE));
                if (ret<0)
                {   rb->splash(HZ*2, true, "Err: wrong data");
                    rb->close(fd); 
                    return 0;
                }
                rb->lcd_set_foreground(LCD_RGBPACK(tempt.Red,tempt.Green,tempt.Blue));
                rb->lcd_drawpixel(x0+x,y0+(bitmap_height-1)-y);
            }
            break;

        case 32: /* 16mil colors + alpha*/
            rb->lseek(fd, (off_t)readlong(&fh.OffBits), SEEK_SET); 
	    for (i = 0; i < bitmap_width * bitmap_height; i++,x++)
            {
                if (x==bitmap_width) { x=0; y++; }
		ret = rb->read(fd, &tempq,sizeof(struct RGBQUAD));
                if (ret<0)
                {   rb->splash(HZ*2, true, "Err: wrong data");
                    rb->close(fd); 
                    return 0;
                }
                rb->lcd_set_foreground(LCD_RGBPACK(tempq.Red,tempq.Green,tempq.Blue));
                rb->lcd_drawpixel(x0+x,y0+(bitmap_height-1)-y);
            }
            break;

    }

    rb->close(fd); 
    rb->lcd_update();
    return 1;
}

int viewer(const struct plugin_api* rb,char* filename)
{
    struct Fileheader fh;
    struct RGBTRIPLE temp;
    int fd,ret,x=0,y=0,x0=0,y0=0,i,zoom=0;
    int bitmap_width, bitmap_height, NEW_WIDTH,NEW_HEIGHT;

    fd = rb->open(filename, O_RDONLY); 
    if (fd<0)
    {   rb->splash(HZ*2, true, "Err: not found");
        return 0;
    }

    rb->read(fd, &fh, sizeof(struct Fileheader));

    bitmap_height = readlong(&fh.Height);
    bitmap_width = readlong(&fh.Width);

    NEW_WIDTH=bitmap_width;
    NEW_HEIGHT=bitmap_height;

    if ((LCD_HEIGHT<bitmap_height)||(LCD_WIDTH<bitmap_width))
    {	if (bitmap_width>bitmap_height)
           zoom=divide(bitmap_width,LCD_WIDTH);
	else
           zoom=divide(bitmap_height,LCD_HEIGHT);
    	NEW_WIDTH=bitmap_width/(zoom+1);
    	NEW_HEIGHT=bitmap_height/(zoom+1);
    }

    /* centering on screen */
    if (NEW_WIDTH<LCD_WIDTH) x0=(LCD_WIDTH-NEW_WIDTH)/2;
    if (NEW_HEIGHT<LCD_HEIGHT) y0=(LCD_HEIGHT-NEW_HEIGHT)/2;

    rb->lseek(fd, (off_t)readlong(&fh.OffBits), SEEK_SET); 

    for (i = 0; (i < NEW_WIDTH * NEW_HEIGHT) ; i++,x++)
    {
	if (x==NEW_WIDTH)
	{ x=0; y++;
	  if (zoom) //skip entire rows
	  	rb->lseek(fd,sizeof(struct RGBTRIPLE)*zoom*bitmap_width, SEEK_CUR);
	}
        ret = rb->read(fd, &temp,sizeof(struct RGBTRIPLE));
	if (ret<0)
        {    rb->splash(HZ*2, true, "Err: wrong data");
	     rb->close(fd); 
             return 0;
        }
	if (zoom) //skip pixels
	     rb->lseek(fd,sizeof(struct RGBTRIPLE)*zoom, SEEK_CUR); 
	rb->lcd_set_foreground(LCD_RGBPACK(temp.Red,temp.Green,temp.Blue));
	rb->lcd_drawpixel(x0+x,y0+(NEW_HEIGHT-1)-y);
    }
    rb->close(fd); 
    rb->lcd_update();
    return 1;
}

