/***************************************************************************
 *             __________               __   ___.
 *   Open      \______   \ ____   ____ |  | _\_ |__   _______  ___
 *   Source     |       _//  _ \_/ ___\|  |/ /| __ \ /  _ \  \/  /
 *   Jukebox    |    |   (  <_> )  \___|    < | \_\ (  <_> > <  <
 *   Firmware   |____|_  /\____/ \___  >__|_ \|___  /\____/__/\_ \
 *                     \/            \/     \/    \/            \/
 * $Id: rockword.c,v 0.2 2005/12/09
 *
 * Copyright (C) 2005 Eli Sherer
 *
 * All files in this archive are subject to the GNU General Public License.
 * See the file COPYING in the source tree root for full license agreement.
 *
 * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
 * KIND, either express or implied.
 *
 ****************************************************************************/
#include "plugin.h"
#include "extras/plugbmp.c"

#ifdef HAVE_LCD_BITMAP //h300 has bitmap
#ifdef HAVE_LCD_COLOR  //h300 has color

/* Palette */
#define COLOR_BLACK        LCD_RGBPACK(0,0,0)
#define COLOR_WHITE        LCD_RGBPACK(255,255,255)
#define COLOR_DARKGRAY     LCD_RGBPACK(128,128,128)
#define COLOR_LIGHTGRAY    LCD_RGBPACK(192,192,192)
#define COLOR_RED          LCD_RGBPACK(128,0,0)
#define COLOR_LIGHTRED     LCD_RGBPACK(255,0,0)
#define COLOR_DARKYELLOW   LCD_RGBPACK(128,128,0)
#define COLOR_YELLOW       LCD_RGBPACK(255,255,0)
#define COLOR_GREEN        LCD_RGBPACK(0,128,0)
#define COLOR_LIGHTGREN    LCD_RGBPACK(0,255,0)
#define COLOR_CYAN         LCD_RGBPACK(0,128,128)
#define COLOR_LIGHTCYAN    LCD_RGBPACK(0,255,255)
#define COLOR_BLUE         LCD_RGBPACK(0,0,128)
#define COLOR_LIGHTBLUE    LCD_RGBPACK(0,0,255)
#define COLOR_PURPLE       LCD_RGBPACK(128,0,128)
#define COLOR_PINK         LCD_RGBPACK(255,0,255)
#define COLOR_BROWN        LCD_RGBPACK(128,64,0)
#define COLOR_LIGHTBROWN   LCD_RGBPACK(255,128,64)

/* pictures */
#define SPLASH_SCREEN PLUGIN_DIR "/rockword/splash.bmp" 
#define BITMAP_TOOLBAR PLUGIN_DIR "/rockword/toolbar.bmp" 
#define BITMAP_KEYB_ENG PLUGIN_DIR "/rockword/keyb-en.bmp" 
#define BITMAP_KEYB_HEB PLUGIN_DIR "/rockword/keyb-he.bmp" 
#define BITMAP_KEYB_CAP PLUGIN_DIR "/rockword/keyb-caps.bmp" 

#define KEYBOARDS 3
#define KYB_ENG 0
#define KYB_CAPS 1
#define KYB_HEB 2

#define KEYS_ACTIONS 14

#define KEY_BACK  0
#define KEY_TAB   1
#define KEY_CAPS  2
#define KEY_CR    3
#define KEY_SHIFT 4
#define KEY_CTRL  5
#define KEY_LANG  6
#define KEY_HOME  7
#define KEY_END   8
#define KEY_DEL   9
#define KEY_UP    10
#define KEY_LEFT  11
#define KEY_DOWN  12
#define KEY_RIGHT 13

#define TOOLBAR_BUTTONS 10
#define TOOLBAR_H 20
#define KEYBOARD_SIZE 62
#define KEYBOARD_Y 95
#define KEYBOARD_H 81
#define ROCKWORD_PALETTE       LCD_RGBPACK(0,64,128)

/* Buttons */
#define ROCKWORD_QUIT   BUTTON_OFF    /* quit program */
#define ROCKWORD_CHOOSE BUTTON_SELECT /* choose button or keyboard on */
#define ROCKWORD_MENU   BUTTON_ON     /* switch between menu and writing */
#define ROCKWORD_CAPS   BUTTON_REC    /* caps lock key */
#define ROCKWORD_KEYB   BUTTON_MODE   /* keyboard on and off */

/* Menus */
#define MENU_NEW 0
#define MENU_LOAD 1
#define MENU_SAVE 2
#define MENU_BOLD 3
#define MENU_ITALIC 4
#define MENU_ULINE 5
#define MENU_COLOR 6
#define MENU_RIGHT 7
#define MENU_CENTER 8
#define MENU_LEFT 9

#define MAX_VIEW_ROWS 17
#define MAX_VIEW_COLS 35

/* Global variables */
static struct plugin_api* rb;

static int curcol=0,currow=0,font_width,font_height,drawcolor=0;
static bool quit=false;

static unsigned rw_colors[18]={
	COLOR_BLACK,COLOR_DARKGRAY,COLOR_RED,COLOR_DARKYELLOW,
	COLOR_GREEN,COLOR_CYAN,COLOR_BLUE,COLOR_PURPLE,COLOR_BROWN,
	COLOR_WHITE,COLOR_LIGHTGRAY,COLOR_LIGHTRED,COLOR_YELLOW,
	COLOR_LIGHTGREN,COLOR_LIGHTCYAN,COLOR_LIGHTBLUE,COLOR_PINK,COLOR_LIGHTBROWN};

static char savebuffer[MAX_VIEW_ROWS][MAX_VIEW_COLS]; /* 19 rows of 36 columns (for now) */
static unsigned stylebuffer[MAX_VIEW_ROWS][MAX_VIEW_COLS]; /* keeps the colors attached to letter */
static short key_coords[KEYBOARD_SIZE][4]=
/* 1st line - 0 */
{{0,0,12}, /* ` */ {11,0,17}, /* 1 */ {27,0,17}, /* 2 */ {43,0,17}, /* 3 */ {59,0,17}, /* 4 */ {75,0,17}, /* 5 */ {91,0,17}, /* 6 */
 {107,0,17}, /* 7 */ {123,0,17}, /* 8 */ {139,0,17}, /* 9 */ {155,0,17}, /* 0 */ {171,0,17}, /* - */ {187,0,17}, /* = */ {203,0,17}, /* <-*/
/* 2nd line - 16 */
 {0,16,21}, /* Tb*/ {20,16,17}, /* q */ {36,16,17}, /* w */ {52,16,17}, /* e */ {68,16,17}, /* r */ {84,16,17}, /* t */ {100,16,17}, /* y */ 
 {116,16,17}, /* u */ {132,16,17}, /* i */ {148,16,17}, /* o */ {164,16,17}, /* p */ {180,16,17}, /* [ */ {196,16,17}, /* ] */ 
/* 3rd line - 32 */
 {0,32,26}, /* Cp*/ {25,32,17}, /* a */ {41,32,17}, /* s */ {57,32,17}, /* d */ {73,32,17}, /* f */ {89,32,17}, /* g */ {105,32,17}, /* h */ 
 {121,32,17}, /* j */ {137,32,17}, /* k */ {153,32,17}, /* l */ {169,32,17}, /* ; */ {185,32,17}, /* ' */ {201,32,19}, /* Cr*/
/* 4th line - 48 */
 {0,48,28}, /* Sh*/ {27,48,17}, /* z */ {43,48,17}, /* x */ {59,48,17}, /* c */ {75,48,17}, /* v */ {91,48,17}, /* b */ {107,48,17}, /* n */
 {123,48,17}, /* m */ {139,48,17}, /* , */ {155,48,17}, /* . */ {171,48,17}, /* / */ {187,48,17}, /* ^ */ {203,48,17}, /* \ */
/* 5th line - 64 */
 {0,64,23}, /* Ct*/ {22,64,17}, /* Ln*/ {38,64,84}, /*   */ {121,64,21}, /* Hm*/ {141,64,17}, /* En*/ {157,64,15}, /* dl*/
 {171,64,17}, /* < */ {187,64,17}, /* V */ {203,64,17}  /* > */ } ;
static unsigned char key_chars[KEYBOARDS][KEYBOARD_SIZE]={
/* English */
{'`','1','2','3','4','5','6','7','8','9','0','-','=',KEY_BACK,
 KEY_TAB,'q','w','e','r','t','y','u','i','o','p','[',']', 
 KEY_CAPS,'a','s','d','f','g','h','j','k','l',';','\'',KEY_CR,
 KEY_SHIFT,'z','x','c','v','b','n','m',',','.','/',KEY_UP,'\\',
 KEY_CTRL,KEY_LANG,' ',KEY_HOME,KEY_END,KEY_DEL,KEY_LEFT,KEY_DOWN,KEY_UP}, 
/* English - CAPS ON */
{'~','!','@','#','$','%','^','&','*','(',')','_','+',KEY_BACK,
 KEY_TAB,'Q','W','E','R','T','Y','U','I','O','P','{','}',
 KEY_CAPS,'A','S','D','F','G','H','J','K','L',':','"',KEY_CR,
 KEY_SHIFT,'Z','X','C','V','B','N','M','<','>','?',KEY_UP,'|',
 KEY_CTRL,KEY_LANG,' ',KEY_HOME,KEY_END,KEY_DEL,KEY_LEFT,KEY_DOWN,KEY_UP},
/* Hebrew */
{';','1','2','3','4','5','6','7','8','9','0','-','=',KEY_BACK,
 KEY_TAB,'/','\'','','','','','','','','',']','[',
 KEY_CAPS,'','','','','','','','','','',',',KEY_CR,
 KEY_SHIFT,'','','','','','','','','','.',KEY_UP,'\\',
 KEY_CTRL,KEY_LANG,' ',KEY_HOME,KEY_END,KEY_DEL,KEY_LEFT,KEY_DOWN,KEY_UP}};

static void init_buffer(void)
{	int i,j;
	for (i=0;i<MAX_VIEW_ROWS;i++)
	{	rb->strcpy(savebuffer[i],"");
		for (j=0;j<MAX_VIEW_COLS;j++) stylebuffer[i][j]=COLOR_BLACK;
        }
}

static void draw_toolbar(void)
{
	loadbitmap(rb,0,0,BITMAP_TOOLBAR);
}

static void draw_palette(bool update)
{
    	int i;

    	rb->lcd_set_foreground(COLOR_LIGHTGRAY);
    	rb->lcd_fillrect(0,LCD_HEIGHT-21,84,21);
    	rb->lcd_set_foreground(COLOR_BLACK);
    	rb->lcd_drawrect(0,LCD_HEIGHT-21,84,21);
    	for (i=0;i<9;i++) /* first set of colors */
	{	rb->lcd_set_foreground(rw_colors[i]);
	        rb->lcd_fillrect(2+i*9,157,7,7);
	        rb->lcd_set_foreground(ROCKWORD_PALETTE);
        	rb->lcd_drawrect(2+i*9,157,7,7);
    	}
    	for (i=0;i<9;i++) /* second set of colors */
    	{   	rb->lcd_set_foreground(rw_colors[i+9]);
        	rb->lcd_fillrect(2+i*9,166,7,7);
        	rb->lcd_set_foreground(ROCKWORD_PALETTE);
		rb->lcd_drawrect(2+i*9,166,7,7);
	}
	if (update) rb->lcd_update();
}


static void inv_cursor(bool update)
{	rb->lcd_set_foreground(COLOR_BLACK);
	rb->lcd_set_drawmode(DRMODE_COMPLEMENT);
	/* cross painting */
	rb->lcd_fillrect(curcol*font_width,TOOLBAR_H+2+currow*font_height,font_width,font_height);
	rb->lcd_set_foreground(rw_colors[drawcolor]);
	rb->lcd_set_drawmode(DRMODE_SOLID);
	if (update) rb->lcd_update();
}

static void inv_palsel(bool update)
{	rb->lcd_set_foreground(ROCKWORD_PALETTE);
	rb->lcd_set_drawmode(DRMODE_COMPLEMENT);
	rb->lcd_drawrect(2+(drawcolor%9)*9,157+(drawcolor>8?9:0),7,7);
	rb->lcd_set_foreground(rw_colors[drawcolor]);
	rb->lcd_set_drawmode(DRMODE_SOLID);
	if (update) rb->lcd_update();
}

static void inv_keysel(int sel,bool update)
{	rb->lcd_set_foreground(ROCKWORD_PALETTE);
	rb->lcd_set_drawmode(DRMODE_COMPLEMENT);
	rb->lcd_fillrect(key_coords[sel][0],KEYBOARD_Y+key_coords[sel][1],key_coords[sel][2],17);
	rb->lcd_set_foreground(rw_colors[drawcolor]);
	rb->lcd_set_drawmode(DRMODE_SOLID);
	if (update) rb->lcd_update();
}

/* static void restore_screen(void)
{

} */

static void clear_text(void)
{	init_buffer();
	rb->lcd_set_foreground(COLOR_WHITE);
	rb->lcd_fillrect(0,TOOLBAR_H,LCD_WIDTH,LCD_HEIGHT-TOOLBAR_H);
	curcol=0; currow=0;
	inv_cursor(false);
	rb->lcd_update();
}

static void update_color(void)
{
	rb->lcd_set_foreground(rw_colors[drawcolor]);
        rb->lcd_fillrect(131,6,10,10);
        rb->lcd_set_foreground(ROCKWORD_PALETTE);
	rb->lcd_drawrect(131,6,10,10);
}

static void pal_loop(void)
{	int done=0,button;

	draw_palette(false);
	inv_palsel(true);

	while (!done)
	{
		button = rb->button_get(true);

		switch(button) 
	        {
	            case BUTTON_UP:
			inv_palsel(false);
	                drawcolor = (drawcolor - 9);
			if (drawcolor<0) drawcolor=9-drawcolor;
			inv_palsel(true);
	                break;
	            case BUTTON_DOWN:
			inv_palsel(false);
	                drawcolor = (drawcolor + 9);
			if (drawcolor>=18) drawcolor-=18;
			inv_palsel(true);
	                break; 
		    case BUTTON_LEFT:
			inv_palsel(false);
			drawcolor = (drawcolor +18 - 1)%18;
			inv_palsel(true);
			break;
		    case BUTTON_RIGHT:
			inv_palsel(false);
			drawcolor = (drawcolor + 1)%18;
			inv_palsel(true);
			break;
		    case ROCKWORD_CHOOSE:
			update_color();
                        done=1;
                	break; 
		    case ROCKWORD_MENU:
		    case ROCKWORD_QUIT:
		    case ROCKWORD_KEYB:
		    case ROCKWORD_CAPS:
			break;
		}/* end switch */
	}/* end while */
	rb->lcd_set_foreground(COLOR_WHITE);
	rb->lcd_fillrect(0,LCD_HEIGHT-21,84,21);
	rb->lcd_update();
}
static void inv_toolsel(int sel,bool update)
{	int tool_coords[TOOLBAR_BUTTONS]={2,20,38,74,92,110,128,164,182,200};
	rb->lcd_set_foreground(ROCKWORD_PALETTE);
	rb->lcd_set_drawmode(DRMODE_COMPLEMENT);
	rb->lcd_drawrect(tool_coords[sel],1,18,18);
	rb->lcd_set_foreground(rw_colors[drawcolor]);
	rb->lcd_set_drawmode(DRMODE_SOLID);
	if (update) rb->lcd_update();
}

static void toolbar_loop(void)
{	int done=0,button,selection=0;

	inv_toolsel(selection,true);

	while (!done)
	{
		button = rb->button_get(true);

		switch(button) 
	        {
		    case BUTTON_LEFT:
			inv_toolsel(selection,false);
			selection = (selection +10 - 1)%10;
			inv_toolsel(selection,true);
			break;
		    case BUTTON_RIGHT:
			inv_toolsel(selection,false);
			selection = (selection + 1)%10;
			inv_toolsel(selection,true);
			break;
		    case ROCKWORD_CHOOSE:
			switch (selection)
			{
				case MENU_NEW:
				break;
				case MENU_LOAD:
				break;
				case MENU_SAVE:
				break;
				case MENU_BOLD:
				break;
				case MENU_ITALIC:
				break;
				case MENU_ULINE:
				break;
				case MENU_COLOR:
					pal_loop();
				break;
				case MENU_RIGHT:
				break;
				case MENU_CENTER:
				break;
				case MENU_LEFT:
				break;
			}
		    case ROCKWORD_MENU:
		    case ROCKWORD_QUIT:
		    case ROCKWORD_KEYB:
		    case ROCKWORD_CAPS:
			inv_toolsel(selection,false);
                        done=1;
			break;
		}/* end switch */
	}/* end while */
	rb->lcd_set_foreground(COLOR_WHITE);
	rb->lcd_fillrect(0,LCD_HEIGHT-21,84,21);
	rb->lcd_update();

}

static void change_keyboard(int *mode,int selection)
{
	inv_keysel(selection,false);
	switch (*mode)
	{
	  case (KYB_ENG):
		*mode=KYB_CAPS;
		loadbitmap(rb,0,KEYBOARD_Y,BITMAP_KEYB_CAP);	
		break;
	  case (KYB_CAPS):
		*mode=KYB_HEB;
		loadbitmap(rb,0,KEYBOARD_Y,BITMAP_KEYB_HEB);	
		break;
	  case (KYB_HEB):
		*mode=KYB_ENG;
		loadbitmap(rb,0,KEYBOARD_Y,BITMAP_KEYB_ENG);	
		break;
	}
	inv_keysel(selection,true);
}

static void keyboard_loop(void)
{	int selection=0,done=0,button,mode=KYB_ENG;
	char temp[3];

	loadbitmap(rb,0,KEYBOARD_Y,BITMAP_KEYB_ENG);
	inv_keysel(selection,true);

	while (!done)
	{
		button = rb->button_get(true);

		switch(button) 
	        {
	            case BUTTON_UP:
                    case BUTTON_UP | BUTTON_REPEAT :
			inv_keysel(selection,false);
	                selection = (selection - 13);
			if (selection<0) selection+=KEYBOARD_SIZE-1;
			inv_keysel(selection,true);
	                break;
	            case BUTTON_DOWN:
                    case BUTTON_DOWN | BUTTON_REPEAT :
			inv_keysel(selection,false);
	                selection = (selection + 13);
			if (selection>=KEYBOARD_SIZE) selection-=KEYBOARD_SIZE-1;
			inv_keysel(selection,true);
	                break; 
		    case BUTTON_LEFT:
                    case BUTTON_LEFT | BUTTON_REPEAT :
			inv_keysel(selection,false);
			selection = (selection - 1);
			if (selection<0) selection=KEYBOARD_SIZE-1;
			inv_keysel(selection,true);
			break;
		    case BUTTON_RIGHT:
                    case BUTTON_RIGHT | BUTTON_REPEAT :
			inv_keysel(selection,false);
			selection = (selection + 1);
			if (selection>=KEYBOARD_SIZE) selection=0;
			inv_keysel(selection,true);
			break;
		    case ROCKWORD_CHOOSE:
			if (key_chars[mode][selection]<KEYS_ACTIONS) /* then action has been pressed */
			{
				switch (key_chars[mode][selection])
				{
                                case KEY_BACK :
                                      inv_cursor(false);
				      curcol--;
				      if (curcol==-1) { curcol=MAX_VIEW_COLS-1; currow--; }
                                      if (currow<0) { currow=0; curcol=0; }
				      savebuffer[currow][curcol]=' ';
				      stylebuffer[currow][curcol]=rw_colors[drawcolor];
				      rb->snprintf(temp,2,"%c",' ');
				      rb->lcd_set_background(COLOR_WHITE);
				      rb->lcd_putsxy(curcol*font_width,TOOLBAR_H+2+currow*font_height,temp);
				      inv_cursor(true);
                                 break; 
                                case KEY_TAB  :
                                 break; 
                                case KEY_CAPS :
                                 break; 
                                case KEY_CR   :
                                 break; 
                                case KEY_SHIFT:
                                 break; 
                                case KEY_CTRL :
                                 break; 
                                case KEY_LANG :
				      change_keyboard(&mode,selection);
                                 break; 
                                case KEY_HOME :
                                      inv_cursor(false);
					curcol=0;
				      inv_cursor(true);
                                 break; 
                                case KEY_END  :
                                      inv_cursor(false);
					curcol=MAX_VIEW_COLS-1;
				      inv_cursor(true);
                                 break; 
                                case KEY_DEL  :
                                      inv_cursor(false);
				      savebuffer[currow][curcol]=' ';
				      rb->snprintf(temp,2,"%c",' ');
				      rb->lcd_set_background(COLOR_WHITE);
				      rb->lcd_putsxy(curcol*font_width,TOOLBAR_H+2+currow*font_height,temp);
				      inv_cursor(true);
                                 break; 
                                case KEY_UP   :
                                      inv_cursor(false);
                                      if (currow!=0) { currow--; }
                                      inv_cursor(true);
                                 break; 
                                case KEY_LEFT :
                                      inv_cursor(false);
				      curcol--;
				      if (curcol==-1) { curcol=MAX_VIEW_COLS-1; currow--; }
                                      if (currow<0) { currow=0; curcol=0; }
                                      inv_cursor(true);
                                 break; 
                                case KEY_DOWN :
                                      inv_cursor(false);
                                      currow++;
                                      inv_cursor(true);
                                 break; 
                                case KEY_RIGHT:
                                      inv_cursor(false);
                                      curcol++;
                                      if (curcol==MAX_VIEW_COLS) { curcol=0; currow++; }
                                      inv_cursor(true);
                                 break; 
                                }
			}
			else
			{	inv_cursor(false);
				savebuffer[currow][curcol]=key_chars[mode][selection];
				rb->snprintf(temp,2,"%c",key_chars[mode][selection]);
				rb->lcd_set_foreground(rw_colors[drawcolor]);
				rb->lcd_set_background(COLOR_WHITE);
				rb->lcd_putsxy(curcol*font_width,TOOLBAR_H+2+currow*font_height,temp);
				if (curcol==MAX_VIEW_COLS) { curcol=0; currow++; }
				else curcol++;
				inv_cursor(true);
			}
                	break; 
		    case ROCKWORD_MENU:
		    case ROCKWORD_QUIT:
		    case ROCKWORD_KEYB:
			done=1;
			break;
		    case ROCKWORD_CAPS:
			change_keyboard(&mode,selection);
			break;
		}/* end switch */
	}/* end while */
	rb->lcd_set_foreground(COLOR_WHITE);
	rb->lcd_fillrect(0,KEYBOARD_Y,LCD_WIDTH,KEYBOARD_H);
	rb->lcd_update();
}

static bool rockword_loop(bool editor)
{   int button=0;

    if (editor)
    while (!quit) {

        button = rb->button_get(true);

        switch(button) 
        {
            case BUTTON_UP:
	    case BUTTON_UP | BUTTON_REPEAT :
                inv_cursor(false);
                if (currow!=0) { currow--; }
                inv_cursor(true);
                break; 
            case BUTTON_LEFT :
	    case BUTTON_LEFT | BUTTON_REPEAT :
                inv_cursor(false);
		curcol--;
		if (curcol==-1) { curcol=MAX_VIEW_COLS-1; currow--; }
                if (currow<0) { currow=0; curcol=0; }
                inv_cursor(true);
                break; 
            case BUTTON_DOWN :
	    case BUTTON_DOWN | BUTTON_REPEAT :
                inv_cursor(false);
                currow++;
                inv_cursor(true);
                break; 
            case BUTTON_RIGHT:
	    case BUTTON_RIGHT | BUTTON_REPEAT :
                inv_cursor(false);
                curcol++;
                if (curcol==MAX_VIEW_COLS) { curcol=0; currow++; }
                inv_cursor(true);
                break;
            case ROCKWORD_QUIT:
                rb->lcd_set_drawmode(DRMODE_SOLID);
                return PLUGIN_OK;
            case ROCKWORD_KEYB:
		keyboard_loop();
                break;
	    case ROCKWORD_MENU:
		toolbar_loop();
		break;
	    case ROCKWORD_CHOOSE:
		break;
	    case ROCKWORD_CAPS:
		break;
            default:
                if (rb->default_event_handler(button) == SYS_USB_CONNECTED)
                    return PLUGIN_USB_CONNECTED;
                break;
        }
        
    } /* end while */
    else button = rb->button_get(true);

    return PLUGIN_OK;
}


enum plugin_status plugin_start(struct plugin_api* api, void* parameter)
{
	/** must have stuff **/
	TEST_PLUGIN_API(api);
	//(void)(parameter);
	rb = api;

    rb->lcd_set_foreground(COLOR_BLACK);
    rb->lcd_fillrect(0,0,LCD_WIDTH,LCD_HEIGHT);
    rb->lcd_set_background(COLOR_WHITE);
    rb->lcd_setfont(FONT_SYSFIXED);
    rb->lcd_getstringsize("A", &font_width, &font_height);

    if (!parameter) /* normal startup */
    {	loadbitmap(rb,0,0,SPLASH_SCREEN);    
    	rb->sleep(HZ*2);
    	rb->lcd_clear_display();
    	draw_toolbar();
	clear_text();
    }
    /* viewer startup */
    /* else
	loadbitmap(rb,0,0,parameter); */
    return rockword_loop(parameter==0);
}

#endif
#endif
